use strict;

die "call with hardware type as argument (bm, ll)\n" if( @ARGV < 1 );

my $hw = @ARGV[0];

print <<EOF;
; Shifter - serial input module for BLINKENmini, LittleLights and BlinkenLeds
; timeout movie data (autogenerated by movie2code.pl for hardware type "$hw")
; version 0.2 date 2006-02-28
; Copyright (C) 2005-2006 Stefan Schuermans <1stein\@schuermans.info>
; Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
; a BlinkenArea project - http://www.blinkenarea.org/



EOF

my $pic_duration;
my @pic;
my $ticks;
my @data;

sub txt2no
{
  my $txt = shift;
  my $no;
  if( $txt =~ /^ *0x([0-9A-Fa-f]+)/ ) {
    $no = hex( $1 );
  }
  elsif( $txt =~ /^ *([0-9]+)/ ) {
    $no = int( $1 );
  }
  else {
    $no = 0;
  }
  return $no;
}

sub read_pic
{
  @pic = ();

  my $line;

  while( $line = <STDIN> ) {
    chomp $line;
    chomp $line;
    if( $line =~ /^ *@([0-9]+)/ ) {
      $pic_duration = $1;
      last;
    }
  }

  if( ! $line ) {
    my $i;
    for( $i = 0; $i < 8; $i++ ) {
      push( @pic, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] );
    }
    return 0;
  }

  while( $line = <STDIN> ) {
    chomp $line;
    chomp $line;
    last if( $line =~ /^ *$/ );
    my @l = split( / +/,  $line );
    $_ = txt2no( $_ ) foreach( @l );
    push( @pic, [@l,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] )
  }
  my $i;
  for( $i = 0; $i < 8; $i++ ) {
    push( @pic, [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0] );
  }

  return 1;
}

sub pic2data_bm
{
  my ($y, $x, $b);
  for( $y = 0; $y < 8; $y++ ) {
    for( $x = 0; $x < 18; $x++ ) {
      for( $b = 0; $b < 3; $b++ ) {
        my $v = @{@pic[$y]}[$x] & 0x20 << $b;
        my ($bit, $byte);
        $bit = $y;
        $byte = $x * 3 + $b;
        @data[$byte] |= 0x01 << $bit if( $v );
      }
    }
  }
}

sub pic2data_ll
{
  my ($y, $x, $b);
  for( $y = 0; $y < 8; $y++ ) {
    for( $x = 0; $x < 18; $x++ ) {
      for( $b = 0; $b < 3; $b++ ) {
        my $v = @{@pic[$y]}[$x] & 0x20 << $b;
        my ($bit, $byte);
        if( $x < 16 ) {
          $bit = $x / 2;
          $byte = ($y + ($x % 2) * 8) * 3 + $b;
        } else {
          $bit = $y;
          $byte = $x * 3 + $b;
        }
        @data[$byte] |= 0x01 << $bit if( $v );
      }
    }
  }
}

sub pic2data
{
  $ticks = int( ($pic_duration + 50) / 100 );

  my $i;
  for( $i = 0; $i < 3*18; $i++ ) {
    @data[$i] = 0;
  }

  if( $hw eq "bm" ) {
    pic2data_bm( );
  } elsif( $hw eq "ll" ) {
    pic2data_ll( );
  }
}

sub data_out
{
  my $byte;
  for( $byte = 0; $byte < 3*18; $byte++ ) {
    print( "        .db     " ) if( $byte % 6 == 0 );
    printf( "0x%02X", @data[$byte] );
    print( ", " ) if( $byte % 6 != 5 );
    print( "\n" ) if( $byte % 6 == 5 );
  }
}

sub convert
{
  read_pic( );

  my ($frame_no, $go_on);
  for( $frame_no = 0, $go_on = 1; $go_on ;$frame_no++ ) {

    print( "; frame $frame_no\n" );

    pic2data( );
    data_out( );

    my $wait = $ticks;
    $wait = 0x01 if( $wait < 0x01 );
    $wait = 0xFF if( $wait > 0xFF );

    $go_on = read_pic( );

    printf( "        .db     0x%02X, 0x%02X                              ; delay %.1f s, %s\n",
            $wait, $go_on, $wait / 10, $go_on ? "next frame follows" : "end" );

  }
  print( "; $frame_no frame(s)\n" );
}

print <<EOF;
; movie data: <frame> [<frame> [...]]
;   frame: <picture> <delay> <end-flag>
;   picture: 18*3 bytes (same format as in OUT_BUF_[12] (see data_*.inc)
;   delay: 1 byte, number of 100ms intervals to wait (0x01..0xFF)
;   end-flag: 0x01 for all frames but the last, 0x00 for last frame
MOVIE_DATA:
EOF

convert;

